#!/usr/bin/env python

# Determine the coordinates of some biomes from BIOME_v4.2,
# in a predefined area of interest

# Data file biome_tab.csv created from the PubTabBiome section
# of the excel file found in
#    http://pmip2.lsce.ipsl.fr/share/synth/biome_v4.2/biome_v4.2.zip
# More details on
#    http://pmip2.lsce.ipsl.fr/synth/vegmaps.shtml

# J-Y Peterschmitt - LSCE - February 2008

from os import path
import Numeric, cdms, MV
from cdms.coord import TransientVirtualAxis

datafile = './biome_tab.csv'
out_dir = './'

# Zone limits
# This is the zone specified in
#    http://pmip2-dev/synth/diagmhprna.shtml
(lat1, lat2) = (0, 30)
(lon1, lon2) = (-20, 30)

# column numbers (starting at ZERO)
ilon = 3 # Column with longitude information
ilat = 4 # Column with latitude information
ibio, exp = 8, 0 # Column with biome type for 0K
#ibio, exp = 9, 6 # Column with biome type for 6K

nbcheck = ibio + 1

nblines = 0
nbskip = 0
nbok = 0
biomeok = 0
skipped_biomes = []

lon_l = []
lat_l = []
desert_l = []
steppe_l = []
savanna_l = []
xerophytic_l = []

# Define the mapping between the biomes we are looking for
# and the biome names found in the data file and defined in
#    http://pmip2.lsce.ipsl.fr/synth/mbmaps.shtml
biome_dic = {}
biome_dic['savanna'] = [savanna_l, ['tropical savanna']]
biome_dic['desert'] = [desert_l, ['desert']]
biome_dic['xerophytic'] = [xerophytic_l, ['tropical xerophytic shrubland',
                                          'temperate xerophytic shrubland',
                                          'xerophytic woods/scrub',
                                          'temperate grassland and xerophytic shrubland']]
biome_dic['steppe'] = [steppe_l, ['steppe']]

biome_list = biome_dic.keys()
biome_set = set(biome_list)

# Extract info from the datafile
print 'Extracting %ik data from %s data file\n' % (exp, datafile)
f_in = open(datafile)
f_in.readline() # Skip first line
for line_in in f_in:
    nblines += 1
    data = line_in.strip().split(';')

    # Silently skip lines that don't have enough data
    if len(data) < nbcheck:
        nbskip += 1
        #print "Not enough data! Skipping line"
        #print data
        continue

    # Skip lines that have no data for the current period
    if not data[ibio]:
        nbskip += 1
        continue

    # Skip lines that are outside of the interest area
    lon = float(data[ilon])
    lat = float(data[ilat])
    if not (lon1 <= lon <= lon2) or not (lat1 <= lat <= lat2):
        nbskip += 1
        continue

    nbok += 1

    # Check if we have found one of the biomes we are looking for
    biome_found = None
    biome_search = data[ibio].lower()
    for biome in biome_list:
        if biome_search in biome_dic[biome][1]:
            biome_found = biome
            break

    # Skip lines where we have not found any interesting biome
    if not biome_found:
        #print 'Skipping biome', biome_search
        if biome_search not in skipped_biomes:
            skipped_biomes.append(biome_search)
        continue

    # Yes, we have found something!
    # NOTE: Check if we already have data for the current lon/lat ??
    biomeok += 1
    print 'Site = %3i  -  Lon = %6.2f  -  Lat = %6.2f  - Biome = %s' % (biomeok, lon, lat, biome)
    lon_l.append(lon)
    lat_l.append(lat)
    biome_dic[biome][0].append(1)
    for nobiome in list(biome_set - set([biome])):
        biome_dic[nobiome][0].append(0)
    
    
    #print data[ilon], data[ilat], data[ibio]

f_in.close()

print 'Processed', nblines, 'lines in the csv data file'
print 'Skipped', nbskip, 'lines outside the area of interest'
print 'Found', nbok, 'lines in the area of interest'
print 'Found', biomeok, 'lines with biomes we were looking for, in the area of interest'
print 'Rejected the following biomes in the area of interest', skipped_biomes
print

# Print summary
for biome in biome_list:
    print biome, len(biome_dic[biome][0]), biome_dic[biome][0]

sites = TransientVirtualAxis("sites", biomeok)
lat = cdms.createVariable(Numeric.asarray(lat_l, Numeric.Float32),
                          axes = [sites],
                          id = 'lat')
lat.units = 'degrees_north'
lon = cdms.createVariable(Numeric.asarray(lon_l, Numeric.Float32),
                          axes = [sites],
                          id = 'lon')
lon.units = 'degrees_east'

# Create the netcdf file
fullpath_out = path.join(out_dir, 'biome_coords_%ik.nc' % (exp,))
f_out = cdms.open(fullpath_out, 'w')
f_out.comment = 'http://pmip2.lsce.ipsl.fr/synth/vegmaps.shtml'
f_out.write(lat)
f_out.write(lon)
for biome in biome_list:
    v = cdms.createVariable(Numeric.array(biome_dic[biome][0], Numeric.Int32),
                            axes = [sites],
                            id = biome)
    f_out.write(v)
f_out.close()

# The end
